-------------------------------------------------------------------------------
-- Title      : Testbench for Ad9512Control.vhd
-- Project    : FMC DEL 1ns 2cha (FFPG)
-- URL        : http://www.ohwr.org/projects/fmc-del-1ns-2cha
-------------------------------------------------------------------------------
-- File       : Ad9512Control_tb.vhd
-- Author(s)  : Jan Pospisil <j.pospisil@cern.ch>
-- Company    : CERN (BE-BI-QP)
-- Created    : 2016-08-18
-- Last update: 2016-08-24
-- Standard   : VHDL'93/02
-------------------------------------------------------------------------------
-- Description: 
-------------------------------------------------------------------------------
-- Copyright (c) 2016 CERN (BE-BI-QP)
-------------------------------------------------------------------------------
-- GNU LESSER GENERAL PUBLIC LICENSE
-------------------------------------------------------------------------------
-- This source file is free software; you can redistribute it and/or modify it
-- under the terms of the GNU Lesser General Public License as published by the
-- Free Software Foundation; either version 2.1 of the License, or (at your
-- option) any later version. This source is distributed in the hope that it
-- will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-- See the GNU Lesser General Public License for more details. You should have
-- received a copy of the GNU Lesser General Public License along with this
-- source; if not, download it from http://www.gnu.org/licenses/lgpl-2.1.html
-------------------------------------------------------------------------------
-- Revisions  :
-- Date        Version  Author
-- 2016-08-24  1.0      Jan Pospisil
------------------------------------------------------------------------------- 

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all; 

entity Ad9512Control_tb is
end entity;

architecture testbench of Ad9512Control_tb is

    constant c_ClkFrequency: positive := 100_000_000;
    constant c_ClkPeriod: time := (1 sec)/real(c_ClkFrequency);
    
    signal Clk_ik, Reset_ir, Cfg_i, Busy_o: std_logic := '0';
    signal ClockSelection_i: std_logic := '0';
    signal ClockRatioMinus1_ib: unsigned(4 downto 0) := to_unsigned(2-1, 5);
    signal SpiAd9512Sclk_o, SpiAd9512Mosi_o, SpiAd9512Miso_i, SpiAd9512Cs_on: std_logic := '0';
    
    subtype t_Ratio is Integer range 1 to 32;

    procedure f_Tick(ticks: in natural) is begin
        wait for ticks * c_ClkPeriod;
    end procedure;

begin

    cDUT: entity work.Ad9512Control(syn)
        generic map (
            g_ClkFrequency => c_ClkFrequency
        )
        port map (
            Clk_ik => Clk_ik,
            Reset_ir => Reset_ir,
            Cfg_i => Cfg_i,
            Busy_o => Busy_o,
            ClockSelection_i => ClockSelection_i,
            ClockRatioMinus1_ib => ClockRatioMinus1_ib,
            SpiAd9512Sclk_o => SpiAd9512Sclk_o,
            SpiAd9512Mosi_o => SpiAd9512Mosi_o,
            SpiAd9512Miso_i => SpiAd9512Miso_i,
            SpiAd9512Cs_on => SpiAd9512Cs_on
        );

    pClk: process is begin
        Clk_ik <= '0';
        wait for c_ClkPeriod/2;
        Clk_ik <= '1';
        wait for c_ClkPeriod/2;
    end process;
    
    pTest: process is
        variable Ratio: t_Ratio;
    begin
        Reset_ir <= '1';
        f_Tick(5);
        Reset_ir <= '0';
        f_Tick(2300);
        
        ClockSelection_i <= '1';
        Ratio := 23;
        ClockRatioMinus1_ib <= to_unsigned(Ratio-1, ClockRatioMinus1_ib'length);
        Cfg_i <= '1';
        f_Tick(1);
        ClockSelection_i <= '0';
        Ratio := 10;
        ClockRatioMinus1_ib <= to_unsigned(Ratio-1, ClockRatioMinus1_ib'length);
        Cfg_i <= '0';
        f_Tick(2300);

        assert false report "NONE. End of simulation." severity failure;
        wait;
    end process;

end architecture;