-------------------------------------------------------------------------------
-- Title      : Testbench for FrequencySense.vhd
-- Project    : FMC DEL 1ns 2cha (FFPG)
-- URL        : http://www.ohwr.org/projects/fmc-del-1ns-2cha
-------------------------------------------------------------------------------
-- File       : FrequencySense_tb.vhd
-- Author(s)  : Jan Pospisil <j.pospisil@cern.ch>
-- Company    : CERN (BE-BI-QP)
-- Created    : 2016-08-23
-- Last update: 2016-08-24
-- Standard   : VHDL'93/02
-------------------------------------------------------------------------------
-- Description: 
-------------------------------------------------------------------------------
-- Copyright (c) 2016 CERN (BE-BI-QP)
-------------------------------------------------------------------------------
-- GNU LESSER GENERAL PUBLIC LICENSE
-------------------------------------------------------------------------------
-- This source file is free software; you can redistribute it and/or modify it
-- under the terms of the GNU Lesser General Public License as published by the
-- Free Software Foundation; either version 2.1 of the License, or (at your
-- option) any later version. This source is distributed in the hope that it
-- will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-- See the GNU Lesser General Public License for more details. You should have
-- received a copy of the GNU Lesser General Public License along with this
-- source; if not, download it from http://www.gnu.org/licenses/lgpl-2.1.html
-------------------------------------------------------------------------------
-- Revisions  :
-- Date        Version  Author
-- 2016-08-24  1.0      Jan Pospisil
------------------------------------------------------------------------------- 

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all; 

entity FrequencySense_tb is
end entity;

architecture testbench of FrequencySense_tb is

    --! computes least possible width of the vector to store value X
    -- http://stackoverflow.com/a/12751341/615627
    function f_log2(x: natural) return natural is
        variable i: natural;
    begin
        i := 0;
        while (2**i < x) loop
            i := i + 1;
        end loop;
        return i;
    end function;
    
    constant c_ClkFrequency: positive := 62_500;
    constant c_ClkPeriod: time := (1 sec)/real(c_ClkFrequency);
    
    constant c_SenseClkFrequency: positive := 5_000;
    constant c_SenseClkPeriod: time := (1 sec)/real(c_SenseClkFrequency);
    
    constant c_SenseFrequencyWidth: positive := f_log2(c_SenseClkFrequency);
    
    signal Clk_ik, SenseClk_k, SenseClk_ik: std_logic := '0';
    signal SenseClkEnable: std_logic := '1';
    signal SenseFrequency_ob: unsigned(c_SenseFrequencyWidth-1 downto 0);
    
    procedure f_Tick(ticks: in natural) is begin
        wait for ticks * c_ClkPeriod;
    end procedure;

begin

    cDUT: entity work.FrequencySense(syn)
        generic map (
            g_ClkFrequency => c_ClkFrequency,
            g_SenseFrequencyWidth => c_SenseFrequencyWidth
        )
        port map (
            Clk_ik => Clk_ik,
            SenseClk_ik => SenseClk_ik,
            SenseFrequency_ob => SenseFrequency_ob
        );

    pClk: process is begin
        Clk_ik <= '0';
        wait for c_ClkPeriod/2;
        Clk_ik <= '1';
        wait for c_ClkPeriod/2;
    end process;
    
    pSenseClk: process is begin
        SenseClk_k <= '0';
        wait for c_SenseClkPeriod/2;
        SenseClk_k <= '1';
        wait for c_SenseClkPeriod/2;
    end process;
    
    SenseClk_ik <= SenseClk_k and SenseClkEnable;
    
    pTest: process is begin
        wait for 5 sec;
        
        SenseClkEnable <= '0';
        wait for 5.35 sec;
        
        SenseClkEnable <= '1';
        wait for 5 sec;
        
        assert false report "NONE. End of simulation." severity failure;
        wait;
    end process;

end architecture;