--
-- Register with variable data length
-- Jan Pospisil
--   2009; KP, FEL, CVUT
--   2015; CTP, ALICE, CERN
--   2016; BE-BI-QP, CERN
--

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity Reg is
	generic (
		g_ResetValue: natural := 0
	);
	port (
		Clk_ik: in std_logic;
		Reset_ir: in std_logic;
		Data_ib: in std_logic_vector;
		Enable_i: in std_logic;
		Data_ob: out std_logic_vector
	);
end entity;

architecture syn of Reg is

    constant c_ResetValue: std_logic_vector(Data_ib'range) := std_logic_vector(to_unsigned(g_ResetValue, Data_ib'length));
    signal Data_b: std_logic_vector(Data_ib'range) := c_ResetValue;

begin

	pReg: process (Clk_ik) is begin
		if rising_edge(Clk_ik) then
			if Reset_ir = '1' then
				Data_b <= c_ResetValue;
			elsif Enable_i = '1' then
				Data_b <= Data_ib;
			end if;
		end if;
	end process;
    
    Data_ob <= Data_b;

end architecture;
