--
-- Shift register
-- Jan Pospisil
--   2010; KP, FEL, CVUT
--   2016; BE-BI-QP, CERN
--

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity ShiftRegister is
	generic (
		g_Width: natural
	);
	port (
		Clk_ik: in std_logic;
		Reset_ir: in std_logic;
		Data_ib: in unsigned(g_Width-1 downto 0);
		Data_ob: out unsigned(g_Width-1 downto 0);
		Shift_i: in std_logic;
		Shift_o: out std_logic;
		ShiftEnable_i: in std_logic;
		LoadEnable_i: in std_logic
	);
end entity;

architecture right of ShiftRegister is

	signal Data_b: unsigned(g_Width-1 downto 0) := (others => '0');

begin

	Data_ob <= Data_b;
	Shift_o <= Data_b(0);

	pShift: process (Clk_ik) is begin
		if rising_edge(Clk_ik) then
            if Reset_ir = '1' then
                Data_b <= (others => '0');
			elsif LoadEnable_i = '1' then
				Data_b <= Data_ib;
			elsif ShiftEnable_i = '1' then
				Data_b <= Shift_i & Data_b(g_Width-1 downto 1);
			end if;
		end if;
	end process;

end architecture;

architecture left of ShiftRegister is

	signal Data_b: unsigned(g_Width-1 downto 0);

begin

	Data_ob <= Data_b;
	Shift_o <= Data_b(g_Width-1);

	pShift: process (Clk_ik) is begin
		if rising_edge(Clk_ik) then
            if Reset_ir = '1' then
                Data_b <= (others => '0');
			elsif LoadEnable_i = '1' then
				Data_b <= Data_ib;
			elsif ShiftEnable_i = '1' then
				Data_b <= Data_b(g_Width-2 downto 0) & Shift_i;
			end if;
		end if;
	end process;

end architecture;