library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all; 

library std;
use std.env.all;
use std.textio.all;

entity pulseGeneratorTime_tb is
end entity;

architecture testbench of pulseGeneratorTime_tb is

    constant CLK_FREQ: real := 200.0e6; -- in Hz
    constant CLK_PERIOD: time := (1.0 sec)/CLK_FREQ;

    constant PULSE_MIN_WIDTH_TIME: time := 20 ns;

    signal clk, reset, signalIn, pulseOut: std_logic := '0';
    signal lastPulseStart, lastPulseWidth: time := 0 ns;
    
    procedure tick(ticks: in natural) is begin
        wait for ticks * CLK_PERIOD;
    end procedure;

begin

    cDUT: entity work.pulseGeneratorTime(syn)
        generic map (
            CLK_FREQ => CLK_FREQ,
            PULSE_MIN_WIDTH_TIME => PULSE_MIN_WIDTH_TIME
        )
        port map (
            clk,
            reset,
            signalIn,
            pulseOut
        );
    
    pClk: process is begin
        clk <= '0';
        wait for CLK_PERIOD/2;
        clk <= '1';
        wait for CLK_PERIOD/2;
    end process;
    
    pPulseWidth: process (pulseOut) is
    begin
        if rising_edge(pulseOut) then
            lastPulseStart <= now;
        else
            lastPulseWidth <= now - lastPulseStart;
            report "Last pulse width: " & time'image(now - lastPulseStart);
        end if;
    end process;
    
    pTest: process is
        variable L: line;
    begin
        -- init wait
        tick(2);
        -- try single output
        signalIn <= '1';
        tick(1);
        signalIn <= '0';
        wait on lastPulseWidth;
        assert lastPulseWidth >= PULSE_MIN_WIDTH_TIME
            report "Pulse is too short! (1)"
            severity failure;
        -- try multiple inputs
        signalIn <= '1';
        tick(1);
        signalIn <= '0';
        wait for PULSE_MIN_WIDTH_TIME/2;
        signalIn <= '1';
        tick(1);
        signalIn <= '0';
        wait on lastPulseWidth;
        assert lastPulseWidth >= PULSE_MIN_WIDTH_TIME
            report "Pulse is too short! (1)"
            severity failure;
        assert false report "NONE. End of simulation." severity failure;
        wait;
    end process;

end architecture;
