//-----------------------------------------------------------------------------
// Title      : UVM testbench for FFPG core
// Project    : FMC DEL 1ns 2cha (FFPG)
// URL        : http://www.ohwr.org/projects/fmc-del-1ns-2cha
//-----------------------------------------------------------------------------
// File       : Testbench.sv
// Author(s)  : Jan Pospisil <j.pospisil@cern.ch>
// Company    : CERN (BE-BI-QP)
// Created    : 2016-06-30
// Last update: 2016-08-24
// Standard   : SystemVerilog, UVM
//-----------------------------------------------------------------------------
// Description: 
//-----------------------------------------------------------------------------
// Copyright (c) 2016 CERN (BE-BI-QP)
//-----------------------------------------------------------------------------
// GNU LESSER GENERAL PUBLIC LICENSE
//-----------------------------------------------------------------------------
// This source file is free software; you can redistribute it and/or modify it
// under the terms of the GNU Lesser General Public License as published by the
// Free Software Foundation; either version 2.1 of the License, or (at your
// option) any later version. This source is distributed in the hope that it
// will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU Lesser General Public License for more details. You should have
// received a copy of the GNU Lesser General Public License along with this
// source; if not, download it from http://www.gnu.org/licenses/lgpl-2.1.html
//-----------------------------------------------------------------------------
// Revisions  :
// Date        Version  Author
// 2016-08-24  1.0      Jan Pospisil
//----------------------------------------------------------------------------- 

`include "timescale.svh"
`include "Config.svh"

import uvm_pkg::*;
import Ad5600Package::*;
import TestbenchPackage::*;

module Testbench;

    parameter c_ClkPeriod = 1e9/`CLK_FREQ; // in [ns]
    
    bit Clk_k, Reset_r;
    bit SpiLoopback;

    FfpgInterface LocalInterface(Clk_k, Reset_r);

    FfpgCoreWrapper #(
        .g_ClkFrequency(`CLK_FREQ)
    ) dut (
        .rst_i(Reset_r),
        .clk_sys_i(Clk_k),
        .wb_adr_i(LocalInterface.Wb.adr),
        .wb_dat_i(LocalInterface.Wb.dat_o),
        .wb_dat_o(LocalInterface.Wb.dat_i),
        .wb_cyc_i(LocalInterface.Wb.cyc),
        .wb_sel_i(LocalInterface.Wb.sel),
        .wb_stb_i(LocalInterface.Wb.stb),
        .wb_we_i(LocalInterface.Wb.we),
        .wb_ack_o(LocalInterface.Wb.ack),
        .wb_err_o(LocalInterface.Wb.err),
        .wb_rty_o(LocalInterface.Wb.rty),
        .wb_stall_o(), // doesn't exist in B3 version of WB
        .wb_int_o(), // doesn't exist in B3 version of WB
        .ClkIn0_ik(LocalInterface.Fmc.ClkIn0_k),
        .TriggerDacFrameSynchronization_n(LocalInterface.TriggerDac.FrameSynchronization_n),
        .TriggerDacSerialClock(LocalInterface.TriggerDac.SerialClock),
        .TriggerDacSerialData(LocalInterface.TriggerDac.SerialData),
        .VcxoDacFrameSynchronization_n(LocalInterface.VcxoDac.FrameSynchronization_n),
        .VcxoDacSerialClock(LocalInterface.VcxoDac.SerialClock),
        .VcxoDacSerialData(LocalInterface.VcxoDac.SerialData),
        .Ch1OutputEnable_o(), // not tested so far
        .Ch2OutputEnable_o(), // not tested so far
        .DelayValue_ob(), // not tested so far
        .Ch1SetLe_o(), // not tested so far
        .Ch1ResLe_o(), // not tested so far
        .Ch2SetLe_o(), // not tested so far
        .Ch2ResLe_o(), // not tested so far
        .Ch1Set_o(LocalInterface.Fmc.Ch1Set),
        .Ch1Res_o(LocalInterface.Fmc.Ch1Res),
        .Ch2Set_o(LocalInterface.Fmc.Ch2Set),
        .Ch2Res_o(LocalInterface.Fmc.Ch2Res),
        .Trigger_i(LocalInterface.Fmc.Trigger),
        .SpiAd9512Sclk_o(), // not tested so far
        .SpiAd9512Mosi_o(SpiLoopback), // not tested so far
        .SpiAd9512Miso_i(SpiLoopback), // not tested so far
        .SpiAd9512Cs_on(), // not tested so far
        .Clk2Sel_o(), // not tested so far
        .Onewire_io(), // not tested so far
        .Led_ob() // not tested so far
    );
    
    initial begin: clockGenerator
        Clk_k = 0;
        forever #(c_ClkPeriod/2) Clk_k = ~Clk_k;
    end

    initial begin: resetGenerator
        Reset_r = 1;
        repeat(3) @(negedge Clk_k);
        Reset_r = 0;
    end
    assign LocalInterface.Wb.rst_i = Reset_r;

    initial begin: mainTest
        // uvm_config_db_options::turn_on_tracing();

        uvm_top.set_report_verbosity_level_hier(UVM_MEDIUM);
        // uvm_top.set_report_verbosity_level_hier(UVM_HIGH);
        // uvm_top.set_report_severity_action_hier(UVM_INFO, UVM_NO_ACTION);
        // uvm_top.set_report_id_action_hier("ja", UVM_NO_ACTION);

        uvm_config_db #(virtual FfpgInterface)::set(null, "uvm_test_top.*", "dut_vi", LocalInterface);
        uvm_top.finish_on_completion = 0; // not to finish when GUI is used

        // run_test("TestTriggerAdc");
        // run_test("TestDelayConfiguration");
        // run_test("TestPulseGenerationBlind");
        // run_test("TestPulseGenerationSequential");
        // run_test("TestPulseGeneration");
        run_test("TestAd9512Communication");

        $stop();
    end
    
    initial begin: timeout
        #100ms $error("Simulation timeout!");
        $stop();
    end

endmodule