#!/usr/bin/env python
# coding: utf8

##-----------------------------------------------------------------------------
## Title      : Python drive: DS18B20 temperature chip driver
## Project    : FMC DEL 1ns 2cha (FFPG)
## URL        : http://www.ohwr.org/projects/fmc-del-1ns-2cha
##-----------------------------------------------------------------------------
## File       : OneWire.py
## Author(s)  : Jan Pospisil <j.pospisil@cern.ch>
## Company    : CERN (BE-BI-QP)
## Created    : 2016-09-06
## Last update: 2016-09-06
## Standard   : Python
##-----------------------------------------------------------------------------
## Description: 
##-----------------------------------------------------------------------------
## Copyright (c) 2016 CERN (BE-BI-QP)
##-----------------------------------------------------------------------------
## GNU LESSER GENERAL PUBLIC LICENSE
##-----------------------------------------------------------------------------
## This source file is free software; you can redistribute it and/or modify it
## under the terms of the GNU Lesser General Public License as published by the
## Free Software Foundation; either version 2.1 of the License, or (at your
## option) any later version. This source is distributed in the hope that it
## will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
## of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
## See the GNU Lesser General Public License for more details. You should have
## received a copy of the GNU Lesser General Public License along with this
## source; if not, download it from http://www.gnu.org/licenses/lgpl-2.1.html
##-----------------------------------------------------------------------------
## Revisions  :
## Date        Version  Author        Comment
## 2016-09-06  1.0      Jan Pospisil  Derived from the initial example driver
##-----------------------------------------------------------------------------

from OneWire import *

class DS18B20(OneWire):
    # # usage:
    # ow = DS18B20(0)
    # print(str(ow.ReadTemperature()) + " °C")

    # TODO: CRC checks

    DS18B20_CMD_ROM_SEARCH = 0xF0
    DS18B20_CMD_ROM_READ = 0x33
    DS18B20_CMD_ROM_MATCH = 0x55
    DS18B20_CMD_ROM_SKIP = 0xCC
    DS18B20_CMD_ALARM_SEARCH = 0xEC
    DS18B20_CMD_CONVERT_T = 0x44
    DS18B20_CMD_WRITE_SCRATCHPAD = 0x4E
    DS18B20_CMD_READ_SCRATCHPAD = 0xBE
    DS18B20_CMD_COPY_SCRATCHPAD = 0x48
    DS18B20_CMD_RECALL_E_2 = 0xB8
    DS18B20_CMD_READ_POWER_SUPPLY = 0xB4

    def __init__(self, fmcSlot = -1, wbOverlay = None, overdrive = False):
        super(DS18B20, self).__init__(fmcSlot, wbOverlay, overdrive)

    def ReadRom(self):
        presence = self.Reset()
        assert presence, "DS18B20 is not present!"
        self.Write(self.DS18B20_CMD_ROM_READ)
        return self.ReadArray(8)
    
    def ReadTemperature(self):
        presence = self.Reset()
        assert presence, "DS18B20 is not present!"
        self.Write(self.DS18B20_CMD_ROM_SKIP) # works only because there is just one device on the bus
        self.Write(self.DS18B20_CMD_CONVERT_T)
        while True:
            if self.ReadBit() == 1:
                break
        presence = self.Reset()
        assert presence, "DS18B20 is not present!"
        self.Write(self.DS18B20_CMD_ROM_SKIP) # works only because there is just one device on the bus
        self.Write(self.DS18B20_CMD_READ_SCRATCHPAD)
        scratchpad = self.ReadArray(9)
        temperature = (scratchpad[0] + (scratchpad[1] << 8)) / 16.0
        return temperature
