#!/usr/bin/env python
# coding: utf8

import sys
from Ffpg import *

def myInt(s):
    try:
        return int(s)
    except:
        return None

def myFloat(s):
    try:
        return float(s)
    except:
        return None

print("FFPG command line interface")
print("")

if len(sys.argv) is not 4 and len(sys.argv) < 7:
    print "Usage:"
    print "  "+sys.argv[0]+" <fmcSlot (LUN)> <channel> <polarity> <pulseWidth> <pulseDelay> <triggerLatency> [<bunches>]"
    print "    Sets and starts selected channel on selected FMC mezzanine"
    print "  or"
    print "  "+sys.argv[0]+" <fmcSlot (LUN)> <channel> stop"
    print "    Stops selected channel on selected FMC mezzanine"
    print ""
    print "  <fmcSlot (LUN)> - 0 | 1"
    print "  <channel> - 1 | 2"
    print "  <polarity> - 0 | 1"
    print "  <pulseWidth> - positive float number in [ns], has to be < LHC bunch period (25 ns)"
    print "  <pulseDelay> - non-negative float number in [ns]"
    print "  <triggerLatency> - non-negative float number in [ns], or -1 (calibration disabled)"
    print "  <bunches> - space separated list of bunch numbers, each 0-3563"
    print "    Can also be range(s) of bunches like: 0-6 10-50"
    print "    If no bunches are provided, no pulses will be generated, but generator will run."
    print "    Output will be in non-active state according to the <polarity>."
    print ""
    sys.exit()

fmcSlot = myInt(sys.argv[1])
channel = myInt(sys.argv[2])
if sys.argv[3] == "stop":
    polarity = "stop"
else:
    polarity = myInt(sys.argv[3])
    pulseWidth = myFloat(sys.argv[4])
    pulseDelay = myFloat(sys.argv[5])
    triggerLatency = myFloat(sys.argv[6])
    bunches = []
    for i in range(7, len(sys.argv)):
        if "-" in sys.argv[i]:
            startBunch, stopBunch = sys.argv[i].split("-", 2)
            bunches = bunches + range(myInt(startBunch), myInt(stopBunch))
        else:
            bunches.append(myInt(sys.argv[i]))

print("Configuration:")
print("  fmcSlot (LUN) = "+str(fmcSlot))
print("  channel = "+str(channel))
if polarity == "stop":
    print("  command = stop")
else:
    if polarity is 1:
        print("  polarity = "+str(polarity)+" (positive pulses)")
    else:
        print("  polarity = "+str(polarity)+" (negative pulses)")
    print("  pulseWidth = "+str(pulseWidth)+" ns")
    print("  pulseDelay = "+str(pulseDelay)+" ns")
    print("  triggerLatency = "+str(triggerLatency)+" ns")
    print("  bunches = "+str(bunches))
print("")

if fmcSlot is not 0 and fmcSlot is not 1:
    raise Exception("fmcSlot (LUN) can be only 0 or 1!");
if channel is not 1 and channel is not 2:
    raise Exception("channel can be only 1 or 2!");
if polarity is not 0 and polarity is not 1 and polarity is not "stop":
    raise Exception("polarity can be only 0 or 1!");
if polarity != "stop":
    if pulseWidth <= 0:
        raise Exception("pulseWidth has to be > 0 ns!");
    if pulseDelay < 0:
        raise Exception("pulseDelay has to be >= 0 ns!");
    if triggerLatency < 0 and triggerLatency != -1:
        raise Exception("triggerLatency has to be >= 0 ns or -1!");
    for bunch in bunches:
        if bunch < 0 or bunch > 3563:
            raise Exception("each bunch number has to be >= 0 and <= 3563!");

def PrintFrequency(pg):
    frequency = pg.GetFrequency()
    if frequency is not None:
        print("RF clock frequency: "+str(frequency/1.0e6) + " MHz")
    else:
        print("RF clock frequency: (unstable)")

# create FFPG driver for FMC slot
pg = Ffpg(fmcSlot)

# activate low-level debug output
# pg.wb.SetDebug(True)

# activate SystemVerilog trace debug
# pg.wb.SetDebugSv("wb_trace.svh")

# stop and disable channel
if polarity == "stop":
    pg.StopChannel(channel)
    pg.DisableChannel(channel)
    print "Done, channel stopped."
    print ""
    sys.exit();

# initialize the driver
# pg.Reset() # this clears and stops both channel effectively inhibiting possibility of both channels running
pg.SelectClock("external", 400.789)
pg.SetRatio(2)
pg.SetOverflow(17820)
pg.Ad9512Sync()
pg.SetTriggerThreshold(0.5)
pg.SetVcxoFrequency(-0.2105) # 125.0000 MHz
# calibration
print("Calibrating channel "+str(channel)+" on FMC slot "+str(fmcSlot)+" to "+str(triggerLatency)+" ns")
if triggerLatency > -1:
    pg.SetTriggerPhase(channel, triggerLatency)
else:
    print("   (Skipping this calibration, no calibration data (-1) provided.)")

# create bunched pulses
pg.ResetBadState(channel)
pg.CreateBunchPulses(channel, pulseDelay, pulseWidth, bunches, polarity)
pg.EnableChannel(channel)
pg.StartChannel(channel)

# print some info
pg.PrintVersion()
# print("Actual temperature: "+str(pg.temp.ReadTemperature()) + " °C")
print("Actual temperature: (temperature read-out disabled - it is too slow)")
PrintFrequency(pg)
pg.PrintControl()
pg.PrintStatus()
pg.PrintDebug()

print ""
print "Done."
print ""
