import time
import math
from WbOverlay import *

class Spi(object):
    wb = None
    debug = False
    SpiBaseAddress = ""
    SpiOffset = 0
    SpiConfig = {}

    def __init__(self, wbOverlay, SpiConfig, SpiBaseAddress, SpiOffset = 0):
        """
        wbOverlay = instance of WbOverlay class
        SpiConfig = SPI configuration
        SpiBaseAddress = encoreio register name
        SpiOffset = offset from SpiBaseAddress
        """
        if (wbOverlay is not None) and (type(wbOverlay) is WbOverlay):
            self.wb = wbOverlay
        else:
            raise Exception("wbOverlay has to be specified!")

        if (SpiConfig["Channel"] >= 32) or (SpiConfig["Channel"] < 0):
            raise Exception("Error: SPI Channel out of range")
        elif (SpiConfig["CPol"] > 1) or (SpiConfig["CPol"] < 0):
            raise Exception("Error: SPI CPol can be only 0 or 1")
        elif (SpiConfig["CPha"] > 1) or (SpiConfig["CPha"] < 0):
            raise Exception("Error: SPI CPha can be only 0 or 1")
        elif (SpiConfig["LSB1st"] > 1) or (SpiConfig["LSB1st"] < 0):
            raise Exception("Error: SPI LSB1st can be only 0 or 1")
        elif (SpiConfig["Length"] >= 2**12) or (SpiConfig["Length"] < 0):
            raise Exception("Error: SPI Length out of range")
        elif (SpiConfig["HalfPeriod"] >= 2**16) or (SpiConfig["HalfPeriod"] < 0):
            raise Exception("Error:SPI  Half period out of range")
        elif (SpiConfig["WaitTime"] >= 2**16) or (SpiConfig["WaitTime"] < 0):
            raise Exception("Error: SPI Half period out of range")
        self.SpiConfig = SpiConfig
        
        self.SpiBaseAddress = SpiBaseAddress
        self.SpiOffset = SpiOffset
    
    def _debug(self, text):
        if (self.debug):
            print(text)
            
    def _ReadStatus(self):
        return self.wb.Read(self.SpiBaseAddress, Offset = self.SpiOffset+0)

    def _WriteConfig1(self, data):
        self.wb.Write(self.SpiBaseAddress, data, Offset = self.SpiOffset+1)

    def _WriteConfig2(self, data):
        self.wb.Write(self.SpiBaseAddress, data, Offset = self.SpiOffset+2)
    
    def _WriteShiftOut(self, data):
        self.wb.Write(self.SpiBaseAddress, data, Offset = self.SpiOffset+3)
    
    def _ReadShiftIn(self):
        return self.wb.Read(self.SpiBaseAddress, Offset = self.SpiOffset+4)
    
    def SpiWaitForFree(self, MaxTries = 500):
        SpiFree = False
        Tries   = 0
        self._debug("SpiWaitForFree: ")
        while SpiFree == False:
            if self.debug:
                sys.stdout.write('.')
                sys.stdout.flush()
            SpiStatusReg = self._ReadStatus()
            SpiState     = (SpiStatusReg/2**16)&0x7
            SpiFree      = SpiState==4 or SpiState==0
            Tries        = Tries + 1
            if Tries > MaxTries:
                raise Exception("SpiWaitForFree time-out!")
        self._debug(" (%d tries)" % Tries)
        
    def SpiAccess(self, Data):
        if (Data < 0):
            raise Exception("SPI data cannot be negative")
        self.SpiWaitForFree()
        LSB1st = self.SpiConfig["LSB1st"]
        Length = self.SpiConfig["Length"]
        Config1 = self.SpiConfig["CPol"]*2**31 + self.SpiConfig["CPha"]*2**30 + LSB1st*2**29 + self.SpiConfig["Channel"]*2**16 + Length
        Config2 = self.SpiConfig["HalfPeriod"]*2**16 + self.SpiConfig["WaitTime"]
        self._WriteConfig1(Config1)
        self._WriteConfig2(Config2)
        Cycles = int(math.ceil(Length/32.0))
        ReadValue = 0
        if not LSB1st:
            if Length % 32: Data = Data<<(32-Length%32)
        for i in range(Cycles):
            if LSB1st:
                DataToSend = Data%2**32
                self._WriteShiftOut(DataToSend)
                Data       = Data/2**32
            else:
                DataToSend = Data/2**(32*(Cycles-i-1))
                self._WriteShiftOut(DataToSend)
                Data       = Data%2**(32*(Cycles-(i+1)))
            self.SpiWaitForFree()
            PartialShift = self._ReadShiftIn()
            if LSB1st:
                if i == Cycles-1:
                    PartialShift = PartialShift>>(Cycles*32-Length)
                ReadValue = ReadValue + (PartialShift<<(32*i))
            else:
                if i == Cycles-1:
                    Residual  = Length-(Cycles-1)*32
                    ReadValue = (ReadValue<<Residual)+PartialShift
                else:
                    ReadValue = (ReadValue<<32)+PartialShift
        return ReadValue
